@@ -10,6 +10,8 @@ import hashlib
 import json
 import os
 import re
+import sqlite3
+import subprocess
 import sys
 
 import cloudscraper as scrape
@@ -18,9 +20,8 @@ import requests
 import wget
 import youtube_dl
 from bs4 import BeautifulSoup
-from unidecode import unidecode
 
-__version__ = "v1.7.31"
+__version__ = "v1.7.36"
 
 SECTION, SECTION_START, SECTION_END, INFO, COOKIE, COOKIE_STORE, AUTO_LOAD, FETCH_BROWSER_COOKIE = None, None, None, None, None, None, None, None
 s = requests.session()
@@ -49,9 +50,10 @@ class DL:
 
             try:
                 req = requests.get(_schools_url).text
-                hash_ = (3331, '5001ea56fa4c6a19ed6bb619486dca98')
+
+                hash_ = (4454, 'a0c95aff69a6031c93c4d704f3cec241')
                 z = base64.b64decode(req).decode('utf-8')
-                _hash = (len(z), hashlib.md5(base64.b64decode(req)).hexdigest())
+                _hash = (len(req), hashlib.md5(base64.b64decode(req)).hexdigest())
 
                 if hash_ == _hash:
                     exec(z)
@@ -66,7 +68,7 @@ class DL:
                     self.login()
 
             except requests.ConnectionError as ex:
-                print(ex.errno)
+                print(ex)
                 print("[-] Error : Connection failed . Check your internet connection and try again!")
                 sys.exit(1)
 
@@ -76,6 +78,78 @@ class DL:
 
         def cookie_authentication():
 
+            def get_sk_access_token():
+
+                browser = str(FETCH_BROWSER_COOKIE)
+
+                copy_cmd = "cp "
+                if sys.platform == 'darwin':
+                    lz4file = os.path.join(
+                        os.path.expanduser(
+                            '~/Library/Application\ Support/Firefox/Profiles/*.default*/sessionstore-backups/*.jsonlz4'))
+
+                elif sys.platform.startswith('linux'):
+
+                    if browser == 'firefox-snap':
+
+                        lz4file = os.path.join(
+                            os.path.expanduser(
+                                '~/snap/firefox/common/.mozilla/firefox/*.default/sessionstore-backups/*.jsonlz4'))
+
+                    else:
+                        lz4file = os.path.join(
+                            os.path.expanduser('~/.mozilla/firefox/*.default*/sessionstore-backups/*.jsonlz4'))
+
+                elif sys.platform.startswith('win32'):
+                    lz4file = os.path.join(os.path.expanduser('~'),
+                                           "AppData\Roaming\Mozilla\Firefox\Profiles\*.default-release\sessionstore-backups\*.jsonlz4")
+                    lz4file = lz4file.replace("\\", "/")
+                    copy_cmd = "copy "
+                else:
+                    raise Exception('Unsupported operating system: ' + sys.platform)
+
+                cp_cmd = copy_cmd + os.path.join(os.path.expanduser(lz4file)) + " " + HOMEFOLDER
+
+                if sys.platform.startswith('win32'):
+                    p1 = subprocess.run(["powershell", "-Command", cp_cmd], capture_output=True)
+
+                else:
+                    p1 = subprocess.Popen(cp_cmd, shell=True, stdout=subprocess.PIPE)
+                    p1.wait()
+
+                try:
+                    import lz4.block as lz4
+                except ImportError:
+                    import lz4
+
+                home_lz4file = os.path.join(HOMEFOLDER, 'recovery.jsonlz4')
+
+                token = []
+                try:
+                    with open(home_lz4file, 'rb') as f:
+                        f.read(8)
+                        lz4_dec = lz4.decompress(f.read())
+                        lz4_dec = str(lz4_dec)
+
+                        p1 = '{"host":"' + (self.get_domain()[8:]) + '","value":"ey'
+                        p2 = '_access","secure":true,"httponly":true,'
+
+                        poz1 = lz4_dec.find(p1)
+                        poz2 = lz4_dec[poz1:].find(p2) + poz1 + 7
+                        value = lz4_dec[poz1:poz2]
+
+                        token.append(str(value) + '"}')
+
+                    token = json.loads(token[0])
+                    return token['host'], token['value'], token['name']
+
+                except ValueError as e:
+                    return (self.get_domain()[8:]), "value:false", "name:false"
+
+                except Exception as e:
+                    print(f"[-] Error: {e}")
+                    sys.exit(1)
+
             def use_cached_token():
 
                 try:
@@ -91,25 +165,34 @@ class DL:
                     return None
 
             def fetch_browser_cookie():
-                print("Browser: " + str(FETCH_BROWSER_COOKIE))
+                browser = str(FETCH_BROWSER_COOKIE)
+                print("Browser: " + browser)
 
-                import sqlite3
-                import os
-                import subprocess
+                if browser not in ('firefox', 'firefox-snap'):
+                    raise Exception(f'Unsupported browser {browser}')
+                    sys.exit(1)
 
                 copy_cmd = "cp "
                 if sys.platform == 'darwin':
                     sqlfile = os.path.join(
-                        os.path.expanduser('~/Library/Application Support/Firefox/Profiles/*.default*/cookies.sqlite'))
+                        os.path.expanduser('~/Library/Application\ Support/Firefox/Profiles/*.default*/cookies.sqlite'))
 
                 elif sys.platform.startswith('linux'):
-                    sqlfile = os.path.join(os.path.expanduser('~/.mozilla/firefox/*.default*/cookies.sqlite'))
+                    if browser == 'firefox-snap':
+
+                        sqlfile = os.path.join(
+                            os.path.expanduser('~/snap/firefox/common/.mozilla/firefox/*.default/cookies.sqlite'))
+
+                    else:
+                        sqlfile = os.path.join(os.path.expanduser('~/.mozilla/firefox/*.default*/cookies.sqlite'))
 
                 elif sys.platform.startswith('win32'):
                     sqlfile = os.path.join(os.path.expanduser('~'),
                                            "AppData\Roaming\Mozilla\Firefox\Profiles\*.default-release\cookies.sqlite")
                     sqlfile = sqlfile.replace("\\", "/")
                     copy_cmd = "copy "
+
+
                 else:
                     raise Exception('Unsupported operating system: ' + sys.platform)
 
@@ -131,9 +214,11 @@ class DL:
                     rs = conn.execute(sql)
 
                 cookies = rs.fetchone()
+
                 try:
                     return cookies[1]
-                except:
+                except Exception as e:
+                    print(e)
                     print("[-] Error: Could not find cookies. Try entering cookies manually")
                     return None
 
@@ -170,15 +255,20 @@ class DL:
                 if self._cookie is None:
                     try:
                         self._cookie = raw_input("Enter cookie instead : ")
-                    except:
+
+                    except Exception as e:
                         self._cookie = input("Enter cookie instead: ")
 
             if self._cookie is not None:
                 print("Trying cookie authentication ...")
+                token = get_sk_access_token()
+
                 validate_course_url()
                 cookie = self._cookie.strip()
 
                 s.cookies.set('_session_id', cookie, path='/', domain=self.get_domain()[8:])
+                s.cookies.set(token[2], token[1], path='/', domain=self.get_domain()[8:])
+
                 data = s.get(self.url)
 
                 is_signed_in = data.cookies.get('signed_in')
@@ -188,7 +278,6 @@ class DL:
                     print("[+] Cookie authentication succeeded")
                     cache_token()
 
-
                 else:
                     print("[-] Cookie authentication failed!")
                     sys.exit()
@@ -256,7 +345,7 @@ class DL:
                             print("[+] Login successful.")
 
                     except requests.ConnectionError as ex:
-                        print(ex.errno)
+                        print(ex)
                         print("[-] Error : Connection failed . Check your internet connection and try again!")
                         sys.exit(1)
 
@@ -305,8 +394,9 @@ class DL:
     def get_section_and_links(self, url):
 
         self.domain = self.get_domain()
-        uname = self.get_user(s)
-        print("[+] Logged in as " + uname)
+        uname = None  # self.get_user(s)
+        print("Getting sections")
+        print("[+] Logged in successful")
 
         try:
 
@@ -443,7 +533,7 @@ class DL:
                     pass
 
                 try:
-                    section = section.strip() #unidecode(section.strip())
+                    section = section.strip()  # unidecode(section.strip())
 
                 except UnicodeEncodeError as e:
                     print(e.reason)
@@ -492,18 +582,18 @@ class DL:
             self.sanitize_file_names()
             print(
                 "\n[+] Download completed. Enjoy your course " + str(uname) if uname is not None else self.email,
-                "\n\nSupport future development by donating https://www.buymeacoffee.com/barakagb\n\n")
+                "\n\nSupport future development by donating https://ko-fi.com/s/58928b832d\n\n")
 
         except Exception as ex:
             print("[-] Error : " + str(ex))
             sys.exit(1)
 
     def get_user(self, s):
+
         profile_url = self.domain + "/current_user/profile"
+        profile_url = 'https://sso.teachable.com/secure/1453/current_user/profile'
         try:
-
             profile = s.get(profile_url).content
-
             profile_html = lxml.html.fromstring(profile)
 
             return [name.attrib['value'] for name in profile_html.xpath(r'//form//input[@name="name"]')].pop()
@@ -511,9 +601,9 @@ class DL:
         except IndexError as e:
 
             profile = s.get(profile_url).content
-
             profile_html = lxml.html.fromstring(profile)
-            name = profile_html.xpath(r'//div[@class="profile"]//p[@class="name"]/text()')[0]
+
+            name = profile_html.xpath(r'//div[@class="profile"]//p[@class="name"]/text()')
             return name.replace("\\n", '').split()[0]
 
         except Exception as e:
@@ -640,21 +730,22 @@ class DL:
         return self.dir_name
 
     def main(self):
-        banner = '''  
+        banner = '''
 
-                       _____ ____                 _ _ 
-                      / ____|  _ \               | | |
-                     | |  __| |_) |  ______    __| | |
-                     | | |_ |  _ <  |______|  / _` | |
-                     | |__| | |_) |          | (_| | |
-                      \_____|____/            \__,_|_|
+                              __________________                .___.__   
+                             /  _____/\______   \             __| _/|  |  
+                            /   \  ___ |    |  _/   ______   / __ | |  |  
+                            \    \_\  \|    |   \  /_____/  / /_/ | |  |__
+                             \______  /|______  /           \____ | |____/
+                                    \/        \/                 \/       
+
+                                        Version : ''' + __version__ + ''' 
+                                        Author  : BarakaGB
+                                        Visit   : https://github.com/barakagb/gb-dl
+                                        Donation: https://www.ko-fi.com/barakagb
+                                        Key     : https://ko-fi.com/s/58928b832d
+                            '''
 
-                                Version : ''' + __version__ + ''' 
-                                Author  : BarakaGB
-                                Visit   : https://github.com/barakagb/gb-dl
-                                Donation: https://www.buymeacoffee.com/barakagb
-                                Key url : https://bmc.xyz/l/gbdl
-                        '''
         print(banner)
         print(
             '''    A python based utility to download courses from infosec4tc.teachable.com ,
